#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
//#include <gdk/gdkx.h>
#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>
#include <xmms/configfile.h>

#include "config.h"
#include "globals.h"
#include "coverview.h"
#include "interface-xmms.h"
#include "graphics.h"
#include "plugconf.h"
#include "filetype.h"

#include "lyrc.h"
#include "lyrc_site.h"

#include "xmms-headers/dock.h"
#include "xmms-headers/widget.h"
#include "xmms-headers/vis.h"
#include "xmms-headers/main.h"
#include "xmms-headers/util.h"

static void coview_init( void );
static void coview_cleanup( void );
static void coview_about( void );
static void coview_configure( void );

gboolean dial = FALSE;

static guint timeout_tag;


void coview_killall(  )
{
	if( !coview_test_visible(  ) )
		coview_window_visible( TRUE );
	xmms_remote_quit( coview_session );
}

void coview_play_prev(  )
{
	xmms_remote_playlist_prev( coview_session );
}

void coview_play_next(  )
{
	xmms_remote_playlist_next( coview_session );
}

void coview_play(  )
{
	xmms_remote_play( coview_session );
}

void coview_pause(  )
{
	xmms_remote_pause( coview_session );
}

void coview_stop(  )
{
	xmms_remote_stop( coview_session );
}

void coview_shuffle(  )
{
	xmms_remote_toggle_shuffle( coview_session );
}

void coview_open(  )
{
	static GtkWidget *filebrowser;
	if( filebrowser != NULL ) {
		gdk_window_raise( filebrowser->window );
		return;
	}
	filebrowser = util_create_filebrowser( TRUE );
	gtk_signal_connect( GTK_OBJECT( filebrowser ), "destroy", GTK_SIGNAL_FUNC( gtk_widget_destroyed ), &filebrowser );
}

gboolean coview_test_shuffle(  )
{
	gboolean test = xmms_remote_is_shuffle( coview_session );
	return test;
}

void coview_repeat(  )
{
	xmms_remote_toggle_repeat( coview_session );
}

gboolean coview_test_repeat(  )
{
	gboolean test = xmms_remote_is_repeat( coview_session );
	return test;
}

gchar *coview_skin(  )
{
	/*
	gchar *temp = xmms_remote_get_skin( coview_session );
	gchar *temp2;
	if( temp )
		temp2 = strdup( temp );
	else
		temp2 = NULL;
	return temp2;
	*/
	return xmms_remote_get_skin( coview_session );
}

gint coview_playlist_length(  )
{
	gint N = xmms_remote_get_playlist_length( coview_session );
	return N;
}

gint coview_playlist_pos(  )
{
	gint N = xmms_remote_get_playlist_pos( coview_session );
	return N;
}

gchar *coview_playlist_filename( gint pos )
{
	gchar *temp = xmms_remote_get_playlist_file( coview_session, pos );
	gchar *temp2;
	if( temp )
		temp2 = strdup( temp );
	else
		temp2 = NULL;
	return temp2;
}

gchar *coview_title( gint pos )
{
	gchar *temp = xmms_remote_get_playlist_title( coview_session, pos );
	gchar *temp2;
	if( temp )
		temp2 = strdup( temp );
	else
		temp2 = NULL;
	return temp2;
}

gboolean coview_test_visible(  )
{
	gboolean test = xmms_remote_is_main_win( coview_session );
	return test;
}

gboolean list_state, eq_state;

void coview_window_visible( gboolean show )
{
	if( show ) {
		xmms_remote_main_win_toggle( coview_session, show );
		xmms_remote_pl_win_toggle( coview_session, list_state );
		xmms_remote_eq_win_toggle( coview_session, eq_state );
	} else {
		xmms_remote_main_win_toggle( coview_session, show );
		list_state = xmms_remote_is_pl_win( coview_session );
		eq_state = xmms_remote_is_eq_win( coview_session );
		xmms_remote_pl_win_toggle( coview_session, FALSE );
		xmms_remote_eq_win_toggle( coview_session, FALSE );
	}
}

void coview_show_yourself( gboolean show )
{
	if( show )
		gtk_widget_show( window );
	else
		gtk_widget_hide( window );
}


/*******************************************************************************
	CONFIGURATION FILES : read, write, write-partial	
*******************************************************************************/


void read_xmms_config(  )
{
	ConfigFile *config;
	gchar *cmd;

	config = xmms_cfg_open_default_file(  );
	if( config ) {
		if( !xmms_cfg_read_string( config, "CDDA", "device", &cdaudio_player ) )
			cdaudio_player = strdup( "/dev/cdrom" );
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "WindowSizeX", &xwin ) )
			xwin = 300;
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "WindowSizeY", &ywin ) )
			ywin = 300;
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "PosX", &posx ) )
			posx = 0;
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "PosY", &posy ) )
			posy = 0;
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "Font", &fontname ) )
			fontname = strdup( "-*-*-*-*-*-*-14-*-*-*-*-*-*" );
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "NoPicture", &empty_picture ) )
			empty_picture = g_strdup_printf( "%s/coverviewer/pingoo.jpg", DATADIR );
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "ImagesDirectory", &image_dir ) )
			image_dir = g_strdup_printf( "%s/.covers/", g_get_home_dir(  ) );
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "ImagesKeywords", &image_keywords ) )
			image_keywords = strdup( "" );
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "TitleFormat", &title_format ) )
			title_format = strdup( "%t (%s : %a)" );
		script_net = g_strdup_printf( "%s/coverviewer/script.py", DATADIR );
		/*if (!xmms_cfg_read_string(config,PLUGIN_NAME,"ScriptNet",&script_net))
		   {script_net=g_strdup_printf("%s/coverviewer/script.py",DATADIR);} */
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "SearchOrder", &search_order ) )
			search_order = strdup( "5124" );
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "GetAllImages", &get_all ) )
			get_all = strdup( "0" );
		if( !xmms_cfg_read_string( config, PLUGIN_NAME, "ScriptAlbum", &script_album ) )
			script_album = g_strdup_printf( "%s/coverviewer/get_album.py", DATADIR );
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "ImageRatioPreserve", &ratio ) )
			ratio = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "SlideShow", &slide ) )
			slide = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "SmartSearch", &smart_search ) )
			smart_search = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "Title", &title_display ) )
			title_display = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "Fullscreen", &fullscreen ) )
			fullscreen = FALSE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "Skin", &skinned ) )
			skinned = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "Buttons", &show_button ) )
			show_button = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "PictureIndex", &show_index ) )
			show_index = TRUE;
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "IndexPosX", &index_x ) )
			index_x = xwin - 37;
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "IndexPosY", &index_y ) )
			index_y = ywin - 30;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "InternetSearch", &internet_search ) )
			internet_search = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "AutoHide", &auto_hide ) )
			auto_hide = FALSE;
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "SlideShowSpeed", &slide_speed ) )
			slide_speed = 1000;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "LyricsEnabled", &lyrics_enabled ) )
			lyrics_enabled = TRUE;
		if( !xmms_cfg_read_boolean( config, PLUGIN_NAME, "LyricsLMBScr", &lyrics_lmbscr ) )
			lyrics_lmbscr = FALSE;
		if( !xmms_cfg_read_string( config, "xmms", "LyricsWebsearchCmd", &lyrics_websearch_cmd ) )
			lyrics_websearch_cmd = "kfmclient exec \"http://google.com/search?q=%a %t lyrics OR text\"";
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "LyricsMaxLineLength", &lyrics_maxlinelength ) )
			lyrics_maxlinelength = 60;
		if( !xmms_cfg_read_int( config, PLUGIN_NAME, "LyricsZoom", &lyrics_zoom ) )
			lyrics_zoom = 0;

		// XMMS 
		if( !xmms_cfg_read_string( config, "xmms", "playlist_font", &lyrics_fontname ) )
			lyrics_fontname = "-*-*-*-*-*-*-10-*-*-*-*-*-*";

		// LYRC
		if( !xmms_cfg_read_int( config, "Lyrc 0.3", "FrontRed", &front_red ) )
			front_red = 0x4300;
		if( !xmms_cfg_read_int( config, "Lyrc 0.3", "FrontBlue", &front_blue ) )
			front_blue = 0x8200;
		if( !xmms_cfg_read_int( config, "Lyrc 0.3", "FrontGreen", &front_green ) )
			front_green = 0xbd00;
		if( !xmms_cfg_read_int( config, "Lyrc 0.3", "BackRed", &back_red ) )
			back_red = 0;
		if( !xmms_cfg_read_int( config, "Lyrc 0.3", "BackBlue", &back_blue ) )
			back_blue = 0;
		if( !xmms_cfg_read_int( config, "Lyrc 0.3", "BackGreen", &back_green ) )
			back_green = 0;

		xmms_cfg_free( config );
		
	} else {
		cdaudio_player = strdup( "/dev/cdrom" );
		xwin = 300;
		ywin = 300;
		fontname = strdup( "-*-*-*-*-*-*-14-*-*-*-*-*-*" );
		title_format = strdup( "%t (%s : %a)" );
		empty_picture = g_strdup_printf( "%s/coverviewer/pingoo.jpg", DATADIR );
		image_dir = g_strdup_printf( "%s/.covers/", g_get_home_dir(  ) );
		image_keywords = strdup( "" );
		script_net = g_strdup_printf( "%s/coverviewer/script.py", DATADIR );
		script_album = g_strdup_printf( "%s/coverviewer/get_album.py", DATADIR );
		search_order = strdup( "5124" );
		get_all = strdup( "0" );
		ratio = TRUE;
		slide = TRUE;
		skinned = TRUE;
		show_button = TRUE;
		show_index = TRUE;
		index_x = xwin - 37;
		index_y = ywin - 30;
		title_display = TRUE;
		fullscreen = FALSE;
		internet_search = TRUE;
		smart_search = TRUE;
		auto_hide = FALSE;
		posx = 0;
		posy = 0;
		lyrics_enabled = TRUE;
		lyrics_lmbscr = FALSE;
		lyrics_fontname = "-*-*-*-*-*-*-10-*-*-*-*-*-*";
		lyrics_websearch_cmd = "kfmclient exec \"http://google.com/search?q=%a %t lyrics OR text\"";
		lyrics_maxlinelength = 60;
		lyrics_zoom = 0;

		front_red = 0x4300;
		front_blue = 0x8200;
		front_green = 0xbd00;
		back_red = 0;
		back_blue = 0;
		back_green = 0;
	}
	
	cmd = g_strdup_printf( "mkdir \"%s\" 2>/dev/null", image_dir );
	system( cmd );
	free( cmd );
	cmd = g_strdup_printf( "mkdir \"%s/.albums\" 2>/dev/null", image_dir );
	system( cmd );
	free( cmd );
	config_loaded = TRUE;
	lyrics_display = FALSE;
	lyrics = g_strdup( "\t\nWelcome\n  to\n    coverviewer lyrics!\n" );
	lyrics_nobr = g_strdup( lyrics );
	lyrics_offset = 0;
	lyrics_offset_h = 0;
	titleheight = 12;
	lineheight = 10;
}

void write_xmms_config(  )
{
	ConfigFile *config;
	config = xmms_cfg_open_default_file(  );
	if( config ) {
		xmms_cfg_write_int( config, PLUGIN_NAME, "WindowSizeX", xwin );
		xmms_cfg_write_int( config, PLUGIN_NAME, "WindowSizeY", ywin );
		gdk_window_get_position( window->window, &posx, &posy );
		xmms_cfg_write_int( config, PLUGIN_NAME, "PosX", posx );
		xmms_cfg_write_int( config, PLUGIN_NAME, "PosY", posy );
		xmms_cfg_write_string( config, PLUGIN_NAME, "Font", fontname );
		xmms_cfg_write_string( config, PLUGIN_NAME, "TitleFormat", title_format );
		xmms_cfg_write_string( config, PLUGIN_NAME, "NoPicture", empty_picture );
		xmms_cfg_write_string( config, PLUGIN_NAME, "ImagesDirectory", image_dir );
		xmms_cfg_write_string( config, PLUGIN_NAME, "ImagesKeywords", image_keywords );
		xmms_cfg_write_string( config, PLUGIN_NAME, "ScriptNet", script_net );
		xmms_cfg_write_string( config, PLUGIN_NAME, "SearchOrder", search_order );
		xmms_cfg_write_string( config, PLUGIN_NAME, "GetAllImages", get_all );
		xmms_cfg_write_string( config, PLUGIN_NAME, "ScriptAlbum", script_album );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "ImageRatioPreserve", ratio );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "SlideShow", slide );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "Title", title_display );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "Fullscreen", fullscreen );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "Skin", skinned );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "Buttons", show_button );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "PictureIndex", show_index );
		xmms_cfg_write_int( config, PLUGIN_NAME, "IndexPosX", index_x );
		xmms_cfg_write_int( config, PLUGIN_NAME, "IndexPosY", index_y );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "InternetSearch", internet_search );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "SmartSearch", smart_search );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "AutoHide", auto_hide );
		xmms_cfg_write_int( config, PLUGIN_NAME, "SlideShowSpeed", slide_speed );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "LyricsEnabled", lyrics_enabled );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "LyricsLMBScr", lyrics_lmbscr );
		xmms_cfg_write_string( config, PLUGIN_NAME, "LyricsWebsearchCmd", lyrics_websearch_cmd );
		xmms_cfg_write_int( config, PLUGIN_NAME, "LyricsMaxLineLength", lyrics_maxlinelength );
		xmms_cfg_write_int( config, PLUGIN_NAME, "LyricsZoom", lyrics_zoom );
		xmms_cfg_write_default_file( config );
		xmms_cfg_free( config );
	}
}

void write_xmms_config_partial(  )
{
	ConfigFile *config;

	config = xmms_cfg_open_default_file(  );
	if( config && !( fullscreen ) ) {
		xmms_cfg_write_int( config, PLUGIN_NAME, "WindowSizeX", xwin );
		xmms_cfg_write_int( config, PLUGIN_NAME, "WindowSizeY", ywin );
		gdk_window_get_position( window->window, &posx, &posy );
		xmms_cfg_write_int( config, PLUGIN_NAME, "PosX", posx );
		xmms_cfg_write_int( config, PLUGIN_NAME, "PosY", posy );
		xmms_cfg_write_boolean( config, PLUGIN_NAME, "PictureIndex", show_index );
		xmms_cfg_write_int( config, PLUGIN_NAME, "IndexPosX", index_x );
		xmms_cfg_write_int( config, PLUGIN_NAME, "IndexPosY", index_y );
		xmms_cfg_write_default_file( config );
		xmms_cfg_free( config );
	}
}

/*******************************************************************************
	Plugin Interface
*******************************************************************************/

GeneralPlugin coview = {
	0,							// Handle, filled in by xmms
	0,							// Filename, filled in by xmms
	0,							// Session ID
	PACKAGE_STRING,				// description
	coview_init,				// Called when plugin is enabled
	coview_about,				// Show the about box
	coview_configure,			// Show the configure box
	coview_cleanup				// Called when plugin is disabled
};

gdouble dx, dy;

void sigpress( GtkWidget * widget, GdkEventButton * event, gpointer callback_data )
{
	gint mx, my;
	GdkModifierType modmask;
	gint nr_pics = ( im_found == 0 ) ? 1 : im_found;

	if( event->button == 3 ) {
		coview_menu(  );
	}

	/* testing the picture index */
	if( show_index &&
		event->y >= index_y && event->y <= index_y + 17 &&
		event->x >= index_x - nr_pics * 22 && event->x <= index_x + 22 ) {
		if( event->button == 2 )
			moving = MOVEINDEX;

		if( event->button == 1 ) {
			if( event->x >= index_x ) {	// clicked lyrics icon
				if( lyrics_enabled ) {
					lyrics_display = TRUE;
					coview_load_lyrics_icon(  );
					coview_draw_lyrics(  );
					if( event->x > index_x + 12 && event->y > index_y + 8 )
						edit_lyrics(  );
				}
			} else { // clicked pic icon
				coview_init_imagebuffer(  );
				if( skinned )
					coview_load_skin( curskin );
				coview_load_image_nr( 1 + ( int ) ( ( index_x - event->x ) / 22 ) );
			}
		}
		return;
	}

	if( lyrics_enabled && event->button == 1 &&
		( ( ( event->y >= 25 && event->y < ywin - 31 &&
			  event->x >= xwin - 26 && event->x <= xwin - 7 ) && skinned ) ||
		  ( ( event->y >= 25 && event->y < ywin - 16 && event->x >= xwin - 10 ) && !skinned ) ) ) {
		// initiate scrollbar scrolling
		moving = SCROLLBAR;
		return;
	}

	if( lyrics_enabled && event->button == 1 &&
		lyrics[0] == '\t' &&
		lyrics_display &&
		( ( skinned && event->y >= 21 + titleheight && event->y <= ywin - 31 && event->x <= xwin - 19 ) || !skinned ) )
	{
		// clicked a "menu" option or a specific lyric from the list
		int clicked_line = ( int ) ( event->y - titleheight - ( skinned ? 21 : 0 ) ) / lineheight;
		if( clicked_line > 3 || lyrics_offset + clicked_line > 3 ) {
			gchar *cmd = get_line( clicked_line + lyrics_offset, lyrics );
			if( strstr( cmd, "[MANAGE]" ) == cmd )
				edit_lyrics(  );
			else if( strstr( cmd, "[SEARCH NEXT SOURCE!]" ) == cmd )
				get_lyrics( play_artist, play_song, NULL, NULL, 0, ++script_nr );
			else if( strstr( cmd, "[SEARCH THE WEB]" ) == cmd )
				lyrics_websearch(  );
			else
				multi_choice( lyrics_offset + clicked_line - 3 );
			return;
		}
	}

	if( event->button == 1 &&
		( ( ( event->y < 21 || event->y > ywin - 31 || event->x < xwin - 24 || event->x > xwin - 17 ) && ( !lyrics_lmbscr || !lyrics_display ) ) ||
		  ( ( event->y < 21 || event->y > ywin - 31 || event->x < 15 ) && lyrics_lmbscr ) )
	  ) {
		// initiate window grab & move
		gdk_window_get_pointer( NULL, &mx, &my, &modmask );
		gdk_window_raise( widget->window );
		if( ( !skinned && event->y <= ywin - 31 ) || ( !show_button && !show_index ) ||
			( skinned &&
			  ( event->y <= ywin - 31 || event->y > ywin - 12 ||
				( event->x < 11 || ( event->x > 125 && event->x < 130 )
				  || ( event->x > 146 && event->x < xwin - 19 - 46 - 28 ) || event->x > xwin - 19 ) )
			  && ( event->x < xwin - 20 || event->y > 12 )
			 ) ) {
			moving = MOVE;
			dx = event->x;
			dy = event->y;
			gdk_pointer_grab( window->window, FALSE,
							  GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, GDK_NONE, GDK_NONE, GDK_CURRENT_TIME );
			dock_move_press( dock_window_list, window, event, FALSE );
		}

		if( event->y >= ywin - 15 && event->x >= xwin - 15 ) {
			// initiate window resize
			moving = RESIZE;
			dx = xwin - mx;
			dy = ywin - my;
			gdk_pointer_grab( window->window, FALSE,
							  GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, GDK_NONE, GDK_NONE, GDK_CURRENT_TIME );
		}
	}

	/* fullscreen on double click */
	if( event->type == GDK_2BUTTON_PRESS && event->button == 1 ) {
		coview_fs_toggle(  );
	}

}


void sigrelease( GtkWidget * widget, GdkEventButton * event, gpointer callback_data )
{
	gint mx, my;
	GdkModifierType modmask;
	gint nr_pics = ( im_found == 0 ) ? 1 : im_found;

	if( event->button == 1 ) {
		gdk_window_get_pointer( NULL, &mx, &my, &modmask );
		gdk_pointer_ungrab( GDK_CURRENT_TIME );

		if( !moving ) {
			if( event->y <= 12 && event->x >= xwin - 19 && event->x <= xwin - 11 && skinned ) {
				/* The user clicked on the maximize button */
				if( index_x > xwin / 2 )
					index_x += ( Xwin - xwin );
				if( index_y > ywin / 2 )
					index_y += ( Ywin - ywin );
				mx = Xwin;
				my = Ywin;
				Xwin = xwin;
				Ywin = ywin;
				xwin = mx;
				ywin = my;
				mx = Posx;
				my = Posy;
				gdk_window_get_position( window->window, &Posx, &Posy );
				gdk_window_move_resize( window->window, mx, my, xwin, ywin );
				coview_init_imagebuffer(  );
				coview_load_skin( curskin );
				coview_load_image( curimage->imagename );
				coview_draw_lyrics(  );
				return;
			}
			if( event->y <= 12 && event->x >= xwin - 10 && event->x <= xwin - 2 && skinned ) {
				/* The user clicked on the mini button */
				if( index_x > xwin / 2 )
					index_x -= ( xwin - 275 );
				if( index_y > ywin / 2 )
					index_y -= ( ywin - 290 );
				xwin = 275;
				ywin = 290;
				Xwin = gdk_screen_width(  );
				Ywin = gdk_screen_height(  );
				gdk_window_get_position( window->window, &posx, &posy );
				Posx = 0;
				Posy = 0;
				if( ( posx + 275 ) < 0 )
					posx = 0;
				if( ( posy + 290 ) < 0 )
					posy = 0;
				gdk_window_move_resize( window->window, posx, posy, xwin, ywin );
				coview_init_imagebuffer(  );
				coview_load_skin( curskin );
				coview_load_image( curimage->imagename );
				coview_draw_lyrics(  );
				return;
			}
			/*if (event->y>=18 && event->y<=ywin-18-37 && event->x>=11 && event->x<=xwin-11-19)
			   {
			   if (slide)
			   {
			   slide_pause=1-slide_pause;
			   }
			   else
			   {
			   curimage=curimage->next;
			   if (!curimage) { curimage=first->next; }
			   if (!curimage) { curimage=first; }
			   coview_load_image(curimage->imagename);
			   }
			   } */
			if( event->y >= ywin - 31 && event->y <= ywin - 12 && skinned && !( show_index && event->y >= index_y && event->y <= index_y + 17 && event->x >= index_x - nr_pics * 22 && event->x <= index_x + 22 ) ) {
				/*testing the control buttons */
				if( show_button && skinned ) {
					if( event->x >= 11 && event->x <= 33 ) {
						coview_play_prev(  );
					}
					if( event->x >= 34 && event->x <= 57 ) {
						coview_play(  );
					}
					if( event->x >= 58 && event->x <= 80 ) {
						coview_pause(  );
					}
					if( event->x >= 81 && event->x <= 103 ) {
						coview_stop(  );
					}
					if( event->x >= 104 && event->x <= 125 ) {
						coview_play_next(  );
					}
				}
				if( show_button && skinned && !show_index ) {
					/* testing open, shuffle and repeat */
					if( event->x >= 130 && event->x <= 146 ) {
						coview_open(  );
					}
					if( event->x >= xwin - 19 - 46 - 28 && event->x <= xwin - 19 - 29 ) {
						coview_shuffle(  );
					}
					if( event->x >= xwin - 19 - 28 && event->x <= xwin - 19 ) {
						coview_repeat(  );
					}
				}
			}
		}

		if( moving == RESIZE ) {
			// finish resizing
			Xwin = gdk_screen_width(  );
			Ywin = gdk_screen_height(  );
			Posx = 0;
			Posy = 0;
			moving = NONE;
			coview_init_imagebuffer(  );
			coview_load_skin( curskin );
			coview_load_image( curimage->imagename );
			coview_draw_lyrics(  );
		}
	}

	if( moving == MOVEINDEX ) {
		imcount = 0;
		redrawskin = TRUE;
		coview_init_imagebuffer(  );
		coview_load_skin( curskin );
		coview_load_image( curimage->imagename );
		coview_draw_lyrics(  );
	}

	/* testing the title area for lyrics display toggle */
	if( lyrics_enabled && event->button == 1 &&
		title_display && moving != SCROLLBAR &&
		( ( event->y >= 16 && event->y <= 34 && skinned ) || ( event->y >= 1 && event->y <= 20 && !skinned ) ) ) {
		moving = NONE;
		if( lyrics_display ) {
			lyrics_display = FALSE;
			coview_init_imagebuffer(  );
			coview_load_image( curimage->imagename );
		} else {
			lyrics_display = TRUE;
			coview_draw_lyrics(  );
		}
	}

	/* lyrics mouse wheel scroll up */
	if( event->button == 4 ) {
		if( lyrics_enabled && lyrics_display ) {
			if( lyrics_offset >= 2 ) {
				lyrics_offset -= 2;
				coview_draw_lyrics(  );
			} else {
				lyrics_display = FALSE;
				coview_init_imagebuffer(  );
				coview_load_skin( curskin );
				coview_load_next_image(  );
			}
		} else {
			coview_init_imagebuffer(  );
			coview_load_skin( curskin );
			coview_load_next_image(  );
		}
	}

	/* lyrics mouse wheel scroll down */
	if( lyrics_enabled && event->button == 5 ) {
		if( lyrics_display ) {
			if( lyrics_offset < count_lines( lyrics ) - ( int ) ( ywin / lineheight / 2 ) * 2 + 8 )
				lyrics_offset += 2;
		} else {
			lyrics_display = TRUE;
		}
		coview_draw_lyrics(  );
	}

	/*if (event->button==2)
	   {
	   show_playlist(coview.xmms_session);
	   coview_song_menu(coview.xmms_session);
	   } */

	moving = NONE;

}

void sigmotion( GtkWidget * widget, GdkEventMotion * event, gpointer callback_data )
{
	gint mx, my;
	GdkModifierType modmask;
	static gdouble last_x = 0, last_y = 0;

	if( lyrics_enabled && lyrics_display &&
		( ( event->state & GDK_BUTTON1_MASK && lyrics_lmbscr ) || event->state & GDK_BUTTON2_MASK ) &&
		( !moving || moving == SCROLLMOUSE ) ) {
		/* lyrics mouse button scroll down */
		if( last_y && event->y - last_y > 0 && event->y - last_y < 5 ) {
			if( lyrics_offset < count_lines( lyrics ) - ( int ) ( ywin / lineheight / 2 ) * 2 + 8 ) {
				lyrics_offset += ( event->y - last_y );
				coview_draw_lyrics(  );
			}
		}
		/* lyrics mouse button scroll up */
		if( last_y && event->y - last_y < 0 && event->y - last_y > -5 ) {
			if( lyrics_offset >= -( event->y - last_y ) ) {
				lyrics_offset -= -( event->y - last_y );
				coview_draw_lyrics(  );
			} else
				lyrics_offset = 0;
		}
		/* lyrics mouse button scroll right */
		if( last_x && event->x - last_x > 0 ) {
			lyrics_offset_h++;
			coview_draw_lyrics(  );
		}
		/* lyrics mouse button scroll left */
		if( last_x && event->x - last_x < 0 )
			if( lyrics_offset_h > 0 ) {
				lyrics_offset_h--;
				coview_draw_lyrics(  );
			}

		last_x = event->x;
		last_y = event->y;
		//XWarpPointer(GDK_WINDOW_XDISPLAY(window->window),None,
		//GDK_WINDOW_XWINDOW(window->window),0,0,0,0,50,50);
		moving = SCROLLMOUSE;
	}

	if( moving == MOVEINDEX ) {
		/* dragging picture index */
		index_x = event->x;
		index_y = event->y;
		if( lyrics_display )
			coview_draw_lyrics(  );
		else
			coview_load_pic_index(  );
	}

	if( lyrics_enabled && moving == SCROLLBAR ) {
		/* lyrics slider scroll down */
		int tmp, skin_top, skin_bottom, lyricsheight, lines = count_lines( lyrics );
		if( skinned ) {
			skin_top = ( title_display ) ? lineheight + 21 : 18;
			skin_bottom = 37;
		} else {
			skin_top = ( title_display ) ? lineheight + 3 : 0;
			skin_bottom = 0;
		}
		lyricsheight = ywin - skin_top - skin_bottom;

		tmp = ( ( event->y - skin_top ) / lyricsheight ) * ( lines - lyricsheight / lineheight + 1 );
		if( tmp < 0 || lines < lyricsheight / lineheight )
			tmp = 0;
		if( tmp > lines - lyricsheight / lineheight + 1 )
			tmp = lines - lyricsheight / lineheight + 1;
		if( tmp != lyrics_offset ) {
			lyrics_offset = tmp;
			coview_draw_lyrics(  );
		}
	}

	if( lyrics_enabled &&
		lyrics_display &&
		lyrics[0] == '\t' &&
		( ( skinned && event->y >= 21 + titleheight && event->y <= ywin - 31 && event->x <= xwin - 19 ) || !skinned ) )
	{
		coview_highlight_lyrics_line( ( int ) ( event->y - titleheight - ( skinned ? 21 : 0 ) ) / lineheight );
	}

	gdk_window_get_pointer( NULL, &mx, &my, &modmask );
	if( moving == MOVE ) {
		gdk_window_move( window->window, mx - dx, my - dy );
		dock_move_motion( window, event );
	}
	if( moving == RESIZE ) {
		if( index_x > xwin / 2 )
			index_x += ( dx + mx - xwin );
		if( index_y > ywin / 2 )
			index_y += ( dy + my - ywin );
		gdk_window_resize( window->window, dx + mx, dy + my );
		xwin = dx + mx;
		ywin = dy + my;
	}
}


void sigkey( GtkWidget * widget, GdkEventKey * event, gpointer callback_data )
{
	if( lyrics_enabled ) {
		/* lyrics arrow key scroll up */
		if( event->keyval == GDK_Up ) {
			if( lyrics_offset >= 2 )
				lyrics_offset -= 2;
		}
		/* lyrics page_up key scroll up */
		if( event->keyval == GDK_Page_Up ) {
			if( lyrics_offset >= 16 )
				lyrics_offset -= 16;
			else if( lyrics_offset >= 2 )
				lyrics_offset = 0;
		}
		/* lyrics arrow key scroll down */
		if( event->keyval == GDK_Down ) {
			if( lyrics_offset < count_lines( lyrics ) - ( int ) ( ywin / lineheight / 2 ) * 2 + 8 )
				lyrics_offset += 2;
		}
		/* lyrics page_down key scroll down */
		if( event->keyval == GDK_Page_Down ) {
			if( lyrics_offset < count_lines( lyrics ) - ( int ) ( ywin / lineheight / 2 ) * 2 + 8 )
				lyrics_offset += 16;
		}
		/* lyrics arrow key scroll right */
		if( event->keyval == GDK_Right ) {
			lyrics_offset_h++;
		}
		/* lyrics arrow key scroll left */
		if( event->keyval == GDK_Left ) {
			if( lyrics_offset_h > 0 )
				lyrics_offset_h--;
		}
		/* lyrics editor */
		if( event->keyval == GDK_m || event->keyval == GDK_M || event->keyval == GDK_e || event->keyval == GDK_E ) {
			edit_lyrics(  );
		}
		/* toggle lyrics/cover */
		if( event->keyval == GDK_l || event->keyval == GDK_L || event->keyval == GDK_t || event->keyval == GDK_T ) {
			if( lyrics_display ) {
				lyrics_display = FALSE;
				coview_init_imagebuffer(  );
				coview_load_image( curimage->imagename );
			} else {
				lyrics_display = TRUE;
				coview_draw_lyrics(  );
			}
		}

		coview_draw_lyrics(  );
	}

	/* config window */
	if( event->keyval == GDK_p || event->keyval == GDK_P ) {
		coview_configure(  );
	}

	/* fullscreen */
	if( event->keyval == GDK_f || event->keyval == GDK_F ) {
		coview_fs_toggle(  );
	}
	
	/* default XMMS commands */
	if( event->keyval == GDK_x || event->keyval == GDK_X ) {
		coview_play(  );
	}
	if( event->keyval == GDK_c || event->keyval == GDK_C ) {
		coview_pause(  );
	}
	if( event->keyval == GDK_v || event->keyval == GDK_V ) {
		coview_stop(  );
	}
	if( event->keyval == GDK_b || event->keyval == GDK_B ) {
		coview_play_next(  );
	}
	if( event->keyval == GDK_z || event->keyval == GDK_Z || event->keyval == GDK_w || event->keyval == GDK_W ) {
		coview_play_prev(  );
	}
	if( event->keyval == GDK_i || event->keyval == GDK_I ) {
		coview_make_folder_icon(  );
	}
}

void sigexpose( GtkWidget * widget, GdkEventExpose * event, gpointer callback_data )
{
	gdk_window_copy_area( drawzone->window,
						  drawzone->style->fg_gc[GTK_STATE_NORMAL],
						  event->area.x, event->area.y,
						  imagebuffer, event->area.x, event->area.y, event->area.width, event->area.height );
}

GeneralPlugin *get_gplugin_info( void )
{
	/* the config window must load/create default
	   value even if plugin is not enabled */
	config_loaded = FALSE;
	return &coview;
}

static void coview_init( void )
{
	gchar *strtemp;

	gtk_init( NULL, NULL );

	lyrc_init(  );

	timeout_tag = gtk_timeout_add( 100, ( GtkFunction ) timeout_func, NULL );
	read_xmms_config(  );
	window = gtk_window_new( GTK_WINDOW_TOPLEVEL );

	dock_add_window( dock_window_list, window );

	drawzone = gtk_drawing_area_new(  );
	gtk_widget_set_events( window,
						   GDK_BUTTON_MOTION_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON2_MASK |
						   GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_KEY_RELEASE_MASK );
	gtk_signal_connect( GTK_OBJECT( window ), "button_press_event", GTK_SIGNAL_FUNC( sigpress ), NULL );
	gtk_signal_connect( GTK_OBJECT( window ), "button_release_event", GTK_SIGNAL_FUNC( sigrelease ), NULL );
	gtk_signal_connect( GTK_OBJECT( window ), "motion_notify_event", GTK_SIGNAL_FUNC( sigmotion ), NULL );
	gtk_signal_connect( GTK_OBJECT( window ), "key_press_event", GTK_SIGNAL_FUNC( sigkey ), NULL );
	// Window signal for repaint signal
	gtk_signal_connect( GTK_OBJECT( drawzone ), "expose_event", GTK_SIGNAL_FUNC( sigexpose ), NULL );

	gtk_widget_realize( window );
	gdk_window_set_decorations( window->window, 0 );
	gdk_window_set_functions( window->window, 8 );
	gdk_window_set_back_pixmap( window->window, NULL, FALSE );

	gtk_container_add( GTK_CONTAINER( window ), drawzone );
	gtk_widget_show( drawzone );
	gtk_widget_show( window );

	init = 0;
	moving = NONE;
	curdir = strdup( "Nowhere" );
	curfile = malloc( sizeof( fileinfo ) );
	curfile->filename = strdup( "Nothing" );
	curfile->title = curfile->artist = curfile->album = curfile->track = curfile->year = NULL;

	curskin = strdup( "Nothing" );
	curstream = strdup( "Nothing" );
	curtitle = ( char * ) malloc( 1 );

	first = malloc( sizeof( image_name_buffer ) );
	first->imagename = NULL;
	first->next = NULL;

	curimage = first;

	Xwin = gdk_screen_width(  );
	Ywin = gdk_screen_height(  );
	Posx = 0;
	Posy = 0;
	if( fullscreen ) {
		gdk_window_move_resize( window->window, 0, 0, Xwin, Ywin );
		Xwin = xwin;
		Ywin = ywin;
		xwin = gdk_screen_width(  );
		ywin = gdk_screen_height(  );
		Posx = posx;
		Posy = posy;
		posx = 0;
		posy = 0;
	} else {
		gdk_window_move_resize( window->window, posx, posy, xwin, ywin );
	}
	coview_init_imagebuffer(  );
	init_temp_img(  );
	write_xmms_config(  );
	from_dir = 2;
	net_search = FALSE;
	strtemp = g_strdup_printf( "\\rm \"%s\"*.lock 2>/dev/null", image_dir );
	system( strtemp );
	free( strtemp );

	//coview_session=coview.xmms_session;
#ifdef __DEBUG__
	printf( "Covervierwer plugin : init\n" );
	printf( "coverviewer build date : %s\n", DATEOFBUILD );
#endif
}

static void coview_cleanup( void )
{
	lyrc_cleanup(  );
	write_xmms_config_partial(  );
	gtk_timeout_remove( timeout_tag );
	// Not really cleaning : we're hiding
	gtk_widget_hide( window );
	clean_temp(  );
#ifdef __DEBUG__
	printf( "Coverview plugin: exiting\n" );
#endif
}

void offabout( GtkWidget * widget )
{
	gtk_widget_destroy( widget );
	dial = FALSE;
}

static void coview_about( void )
{
	GtkWidget *dialog, *label, *okay_button, *hbox;
	gchar *text;

	if( !dial ) {
		//??? read_xmms_config(  );
		dial = TRUE;
		dialog = gtk_dialog_new(  );
		text = g_strdup_printf( "XMMS CoverViewerXP (%s)\n\"Images And Words\"\n\n"
								"by Caligari <caligari77@users.sourceforge.net>\n"
								"and camico <camico users.sourceforge.net>", VERSION );
		label = gtk_label_new( text );
		okay_button = gtk_button_new_with_label( "OK" );
		gtk_signal_connect_object( GTK_OBJECT( okay_button ), "clicked", GTK_SIGNAL_FUNC( offabout ),
								   ( GtkObject * ) dialog );
		gtk_signal_connect_object( GTK_OBJECT( dialog ), "destroy", GTK_SIGNAL_FUNC( offabout ),
								   ( GtkObject * ) dialog );

		hbox = gtk_hbox_new( FALSE, 10 );
		gtk_box_pack_start( GTK_BOX( hbox ), label, FALSE, FALSE, 10 );
		gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog )->vbox ), hbox, FALSE, FALSE, 10 );
		gtk_container_add( GTK_CONTAINER( GTK_DIALOG( dialog )->action_area ), okay_button );
		gtk_widget_show_all( dialog );
		gtk_window_set_title( ( GtkWindow * ) dialog, ( gchar * ) "CoverViewer plugin" );
		gtk_window_set_policy( GTK_WINDOW( dialog ), FALSE, FALSE, FALSE );
	}
}

static void coview_configure( void )
{
	if( !conf ) {
		conf = TRUE;
		conf_window(  );
	}
}
